@@ -2,8 +2,9 @@
  * towfc_s.c
  *
  * September 2017, Reini Urban
+ * January 2022, Reini Urban
  *
- * Copyright (c) 2017 by Reini Urban
+ * Copyright (c) 2017,2022 by Reini Urban
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
@@ -42,14 +43,14 @@
  *    \b iswfc() checks the uppercase character for a mapping to foldcase,
  *    returning the number of new wide character codepoints needed.
  *    The usual \c iswupper(wc) case returns 1, and the special 104 full
- * folding cases as specified in Unicode 10.0 \c CaseFolding.txt return either
- * 2 or 3.
+ *    folding cases as specified in Unicode 10.0 \c CaseFolding.txt return either
+ *    2 or 3. This implements Unicode 14.0
  *
  * @param[in]   wc  unicode character codepoint
  *
  * @retval  0   when there is no corresponding lowercase mapping.
  * @retval  1   when there is a corresponding common or simple foldcase
- * mapping.
+ *              mapping.
  * @retval  2   a full case-folding expands to 2 characters
  * @retval  3   a full case-folding expands to 3 characters
  *
@@ -68,6 +69,7 @@ static const struct {
     unsigned short lower1;
     unsigned short lower2;
 } tbl2[] = {
+    // clang-format off
     {0x00DF, 0x0073, 0x0073}, /* LATIN SMALL LETTER SHARP S => ss */
     {0x0130, 0x0069, 0x0307}, /* LATIN CAPITAL LETTER I WITH DOT ABOVE */
     {0x0149, 0x02BC, 0x006E}, /* LATIN SMALL LETTER N PRECEDED BY APOSTROPHE */
@@ -81,10 +83,8 @@ static const struct {
     {0x1E9E, 0x0073, 0x0073}, /* LATIN CAPITAL LETTER SHARP S => ss */
     {0x1F50, 0x03C5, 0x0313}, /* GREEK SMALL LETTER UPSILON WITH PSILI */
     /* the following folding mappings need to be normalized to NFD */
-    {0x1F80, 0x1F00,
-     0x03B9}, /* GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI */
-    {0x1F81, 0x1F01,
-     0x03B9}, /* GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI */
+    {0x1F80, 0x1F00, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI */
+    {0x1F81, 0x1F01, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI */
     {0x1F82, 0x1F02, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA
                                  AND YPOGEGRAMMENI */
     {0x1F83, 0x1F03, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA
@@ -97,10 +97,8 @@ static const struct {
                                  PERISPOMENI AND YPOGEGRAMMENI */
     {0x1F87, 0x1F07, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH DASIA AND
                                  PERISPOMENI AND YPOGEGRAMMENI */
-    {0x1F88, 0x1F00,
-     0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI */
-    {0x1F89, 0x1F01,
-     0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI */
+    {0x1F88, 0x1F00, 0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI */
+    {0x1F89, 0x1F01, 0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI */
     {0x1F8A, 0x1F02, 0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND
                                  VARIA AND PROSGEGRAMMENI */
     {0x1F8B, 0x1F03, 0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND
@@ -113,10 +111,8 @@ static const struct {
                                  PERISPOMENI AND PROSGEGRAMMENI */
     {0x1F8F, 0x1F07, 0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND
                                  PERISPOMENI AND PROSGEGRAMMENI */
-    {0x1F90, 0x1F20,
-     0x03B9}, /* GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI */
-    {0x1F91, 0x1F21,
-     0x03B9}, /* GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI */
+    {0x1F90, 0x1F20, 0x03B9}, /* GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI */
+    {0x1F91, 0x1F21, 0x03B9}, /* GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI */
     {0x1F92, 0x1F22, 0x03B9}, /* GREEK SMALL LETTER ETA WITH PSILI AND VARIA
                                  AND YPOGEGRAMMENI */
     {0x1F93, 0x1F23, 0x03B9}, /* GREEK SMALL LETTER ETA WITH DASIA AND VARIA
@@ -129,10 +125,8 @@ static const struct {
                                  PERISPOMENI AND YPOGEGRAMMENI */
     {0x1F97, 0x1F27, 0x03B9}, /* GREEK SMALL LETTER ETA WITH DASIA AND
                                  PERISPOMENI AND YPOGEGRAMMENI */
-    {0x1F98, 0x1F20,
-     0x03B9}, /* GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI */
-    {0x1F99, 0x1F21,
-     0x03B9}, /* GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI */
+    {0x1F98, 0x1F20, 0x03B9}, /* GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI */
+    {0x1F99, 0x1F21, 0x03B9}, /* GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI */
     {0x1F9A, 0x1F22, 0x03B9}, /* GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA
                                  AND PROSGEGRAMMENI */
     {0x1F9B, 0x1F23, 0x03B9}, /* GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA
@@ -145,10 +139,8 @@ static const struct {
                                  PERISPOMENI AND PROSGEGRAMMENI */
     {0x1F9F, 0x1F27, 0x03B9}, /* GREEK CAPITAL LETTER ETA WITH DASIA AND
                                  PERISPOMENI AND PROSGEGRAMMENI */
-    {0x1FA0, 0x1F60,
-     0x03B9}, /* GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI */
-    {0x1FA1, 0x1F61,
-     0x03B9}, /* GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI */
+    {0x1FA0, 0x1F60, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI */
+    {0x1FA1, 0x1F61, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI */
     {0x1FA2, 0x1F62, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA
                                  AND YPOGEGRAMMENI */
     {0x1FA3, 0x1F63, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA
@@ -161,10 +153,8 @@ static const struct {
                                  PERISPOMENI AND YPOGEGRAMMENI */
     {0x1FA7, 0x1F67, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH DASIA AND
                                  PERISPOMENI AND YPOGEGRAMMENI */
-    {0x1FA8, 0x1F60,
-     0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI */
-    {0x1FA9, 0x1F61,
-     0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI */
+    {0x1FA8, 0x1F60, 0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI */
+    {0x1FA9, 0x1F61, 0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI */
     {0x1FAA, 0x1F62, 0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND
                                  VARIA AND PROSGEGRAMMENI */
     {0x1FAB, 0x1F63, 0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND
@@ -177,33 +167,25 @@ static const struct {
                                  PERISPOMENI AND PROSGEGRAMMENI */
     {0x1FAF, 0x1F67, 0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND
                                  PERISPOMENI AND PROSGEGRAMMENI */
-    {0x1FB2, 0x1F70,
-     0x03B9}, /* GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI */
+    {0x1FB2, 0x1F70, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI */
     {0x1FB3, 0x03B1, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI */
-    {0x1FB4, 0x03AC,
-     0x03B9}, /* GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI */
+    {0x1FB4, 0x03AC, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI */
     {0x1FB6, 0x03B1, 0x0342}, /* GREEK SMALL LETTER ALPHA WITH PERISPOMENI */
-    {0x1FBC, 0x03B1,
-     0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI */
-    {0x1FC2, 0x1F74,
-     0x03B9}, /* GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI */
+    {0x1FBC, 0x03B1, 0x03B9}, /* GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI */
+    {0x1FC2, 0x1F74, 0x03B9}, /* GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI */
     {0x1FC3, 0x03B7, 0x03B9}, /* GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI */
-    {0x1FC4, 0x03AE,
-     0x03B9}, /* GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI */
+    {0x1FC4, 0x03AE, 0x03B9}, /* GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI */
     {0x1FC6, 0x03B7, 0x0342}, /* GREEK SMALL LETTER ETA WITH PERISPOMENI */
     {0x1FCC, 0x03B7, 0x03B9}, /* GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI */
     {0x1FD6, 0x03B9, 0x0342}, /* GREEK SMALL LETTER IOTA WITH PERISPOMENI */
     {0x1FE4, 0x03C1, 0x0313}, /* GREEK SMALL LETTER RHO WITH PSILI */
     {0x1FE6, 0x03C5, 0x0342}, /* GREEK SMALL LETTER UPSILON WITH PERISPOMENI */
-    {0x1FF2, 0x1F7C,
-     0x03B9}, /* GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI */
+    {0x1FF2, 0x1F7C, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI */
     {0x1FF3, 0x03C9, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI */
-    {0x1FF4, 0x03CE,
-     0x03B9}, /* GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI */
+    {0x1FF4, 0x03CE, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI */
     /* eof nfd */
     {0x1FF6, 0x03C9, 0x0342}, /* GREEK SMALL LETTER OMEGA WITH PERISPOMENI */
-    {0x1FFC, 0x03C9,
-     0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI */
+    {0x1FFC, 0x03C9, 0x03B9}, /* GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI */
     {0xFB00, 0x0066, 0x0066}, /* LATIN SMALL LIGATURE FF */
     {0xFB01, 0x0066, 0x0069}, /* LATIN SMALL LIGATURE FI */
     {0xFB02, 0x0066, 0x006C}, /* LATIN SMALL LIGATURE FL */
@@ -214,7 +196,9 @@ static const struct {
     {0xFB15, 0x0574, 0x056B}, /* ARMENIAN SMALL LIGATURE MEN INI */
     {0xFB16, 0x057E, 0x0576}, /* ARMENIAN SMALL LIGATURE VEW NOW */
     {0xFB17, 0x0574, 0x056D}, /* ARMENIAN SMALL LIGATURE MEN XEH */
-    {0, 0, 0}};
+    {0, 0, 0}
+    // clang-format on
+};
 
 static const struct {
     unsigned short upper;
@@ -222,37 +206,26 @@ static const struct {
     unsigned short lower2;
     unsigned short lower3;
 } tbl3[] = {
-    {0x0390, 0x03B9, 0x0308,
-     0x0301}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
-    {0x03B0, 0x03C5, 0x0308,
-     0x0301}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
-    {0x1F52, 0x03C5, 0x0313,
-     0x0300}, /* GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA */
-    {0x1F54, 0x03C5, 0x0313,
-     0x0301}, /* GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA */
-    {0x1F56, 0x03C5, 0x0313,
-     0x0342}, /* GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI */
-    {0x1FB7, 0x03B1, 0x0342,
-     0x03B9}, /* GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI */
-    {0x1FC7, 0x03B7, 0x0342,
-     0x03B9}, /* GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI */
-    {0x1FD2, 0x03B9, 0x0308,
-     0x0300}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA */
-    {0x1FD3, 0x03B9, 0x0308,
-     0x0301}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA */
-    {0x1FD7, 0x03B9, 0x0308,
-     0x0342}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI */
-    {0x1FE2, 0x03C5, 0x0308,
-     0x0300}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA */
-    {0x1FE3, 0x03C5, 0x0308,
-     0x0301}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA */
-    {0x1FE7, 0x03C5, 0x0308,
-     0x0342}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI */
-    {0x1FF7, 0x03C9, 0x0342,
-     0x03B9}, /* GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI */
+    // clang-format off
+    {0x0390, 0x03B9, 0x0308, 0x0301}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
+    {0x03B0, 0x03C5, 0x0308, 0x0301}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
+    {0x1F52, 0x03C5, 0x0313, 0x0300}, /* GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA */
+    {0x1F54, 0x03C5, 0x0313, 0x0301}, /* GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA */
+    {0x1F56, 0x03C5, 0x0313, 0x0342}, /* GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI */
+    {0x1FB7, 0x03B1, 0x0342, 0x03B9}, /* GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI */
+    {0x1FC7, 0x03B7, 0x0342, 0x03B9}, /* GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI */
+    {0x1FD2, 0x03B9, 0x0308, 0x0300}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA */
+    {0x1FD3, 0x03B9, 0x0308, 0x0301}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA */
+    {0x1FD7, 0x03B9, 0x0308, 0x0342}, /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI */
+    {0x1FE2, 0x03C5, 0x0308, 0x0300}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA */
+    {0x1FE3, 0x03C5, 0x0308, 0x0301}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA */
+    {0x1FE7, 0x03C5, 0x0308, 0x0342}, /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI */
+    {0x1FF7, 0x03C9, 0x0342, 0x03B9}, /* GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI */
     {0xFB03, 0x0066, 0x0066, 0x0069}, /* LATIN SMALL LIGATURE FFI */
     {0xFB04, 0x0066, 0x0066, 0x006C}, /* LATIN SMALL LIGATURE FFL */
-    {0, 0, 0, 0}};
+    {0, 0, 0, 0}
+    // clang-format on
+};
 
 /* Return the number of wide lower-case characters needed to full fold-case
    the given uppercase character. Returns 0, 1, 2 or 3.
